@@ -36,8 +36,7 @@
  *
  */
 
-#include <string.h>
-
+#include <cstring>
 #include <string>
 #include <vector>
 
@@ -62,7 +61,8 @@ class APNGEncoder : public Encoder {
   std::vector<JxlPixelFormat> AcceptedFormats() const override {
     std::vector<JxlPixelFormat> formats;
     for (const uint32_t num_channels : {1, 2, 3, 4}) {
-      for (const JxlDataType data_type : {JXL_TYPE_UINT8, JXL_TYPE_UINT16}) {
+      for (const JxlDataType data_type :
+           {JXL_TYPE_UINT8, JXL_TYPE_UINT16, JXL_TYPE_FLOAT}) {
         for (JxlEndianness endianness : {JXL_BIG_ENDIAN, JXL_LITTLE_ENDIAN}) {
           formats.push_back(
               JxlPixelFormat{num_channels, data_type, endianness, /*align=*/0});
@@ -73,17 +73,30 @@ class APNGEncoder : public Encoder {
   }
   Status Encode(const PackedPixelFile& ppf, EncodedImage* encoded_image,
                 ThreadPool* pool) const override {
+    // Encode main image frames
     JXL_RETURN_IF_ERROR(VerifyBasicInfo(ppf.info));
     encoded_image->icc.clear();
     encoded_image->bitstreams.resize(1);
-    return EncodePackedPixelFileToAPNG(ppf, pool,
-                                       &encoded_image->bitstreams.front());
+    JXL_RETURN_IF_ERROR(EncodePackedPixelFileToAPNG(
+        ppf, pool, &encoded_image->bitstreams.front()));
+
+    // Encode extra channels
+    for (size_t i = 0; i < ppf.extra_channels_info.size(); ++i) {
+      encoded_image->extra_channel_bitstreams.emplace_back();
+      auto& ec_bitstreams = encoded_image->extra_channel_bitstreams.back();
+      ec_bitstreams.emplace_back();
+      JXL_RETURN_IF_ERROR(EncodePackedPixelFileToAPNG(
+          ppf, pool, &ec_bitstreams.back(), true, i));
+    }
+    return true;
   }
 
  private:
   Status EncodePackedPixelFileToAPNG(const PackedPixelFile& ppf,
                                      ThreadPool* pool,
-                                     std::vector<uint8_t>* bytes) const;
+                                     std::vector<uint8_t>* bytes,
+                                     bool encode_extra_channels = false,
+                                     size_t extra_channel_index = 0) const;
 };
 
 void PngWrite(png_structp png_ptr, png_bytep data, png_size_t length) {
@@ -127,9 +140,14 @@ class BlobsWriterPNG {
   }
 
  private:
+  // TODO(eustas): use array
   static JXL_INLINE char EncodeNibble(const uint8_t nibble) {
-    JXL_ASSERT(nibble < 16);
-    return (nibble < 10) ? '0' + nibble : 'a' + nibble - 10;
+    if (nibble < 16) {
+      return (nibble < 10) ? '0' + nibble : 'a' + nibble - 10;
+    } else {
+      JXL_DEBUG_ABORT("Internal logic error");
+      return 0;
+    }
   }
 
   static Status EncodeBase16(const std::string& type,
@@ -137,7 +155,7 @@ class BlobsWriterPNG {
                              std::vector<std::string>* strings) {
     // Encoding: base16 with newline after 72 chars.
     const size_t base16_size =
-        2 * bytes.size() + DivCeil(bytes.size(), size_t(36)) + 1;
+        2 * bytes.size() + DivCeil(bytes.size(), static_cast<size_t>(36)) + 1;
     std::string base16;
     base16.reserve(base16_size);
     for (size_t i = 0; i < bytes.size(); ++i) {
@@ -146,7 +164,7 @@ class BlobsWriterPNG {
       base16.push_back(EncodeNibble(bytes[i] & 0x0F));
     }
     base16.push_back('\n');
-    JXL_ASSERT(base16.length() == base16_size);
+    JXL_ENSURE(base16.length() == base16_size);
 
     char key[30];
     snprintf(key, sizeof(key), "Raw profile type %s", type.c_str());
@@ -155,7 +173,7 @@ class BlobsWriterPNG {
     snprintf(header, sizeof(header), "\n%s\n%8" PRIuS, type.c_str(),
              bytes.size());
 
-    strings->push_back(std::string(key));
+    strings->emplace_back(key);
     strings->push_back(std::string(header) + base16);
     return true;
   }
@@ -246,14 +264,12 @@ void MaybeAddGAMA(const JxlColorEncoding& c_enc, png_structp png_ptr,
 void MaybeAddCLLi(const JxlColorEncoding& c_enc, const float intensity_target,
                   png_structp png_ptr, png_infop info_ptr) {
   if (c_enc.transfer_function != JXL_TRANSFER_FUNCTION_PQ) return;
+  if (intensity_target == 10'000) return;
 
-  const uint32_t max_cll =
-      static_cast<uint32_t>(10000.f * Clamp1(intensity_target, 0.f, 10000.f));
+  const uint32_t max_content_light_level =
+      static_cast<uint32_t>(10'000.f * Clamp1(intensity_target, 0.f, 10'000.f));
   png_byte chunk_data[8] = {};
-  chunk_data[0] = (max_cll >> 24) & 0xFF;
-  chunk_data[1] = (max_cll >> 16) & 0xFF;
-  chunk_data[2] = (max_cll >> 8) & 0xFF;
-  chunk_data[3] = max_cll & 0xFF;
+  png_save_uint_32(chunk_data, max_content_light_level);
   // Leave MaxFALL set to 0.
   png_unknown_chunk chunk;
   memcpy(chunk.name, "cLLi", 5);
@@ -266,15 +282,21 @@ void MaybeAddCLLi(const JxlColorEncoding& c_enc, const float intensity_target,
 }
 
 Status APNGEncoder::EncodePackedPixelFileToAPNG(
-    const PackedPixelFile& ppf, ThreadPool* pool,
-    std::vector<uint8_t>* bytes) const {
-  size_t xsize = ppf.info.xsize;
-  size_t ysize = ppf.info.ysize;
-  bool has_alpha = ppf.info.alpha_bits != 0;
-  bool is_gray = ppf.info.num_color_channels == 1;
-  size_t color_channels = ppf.info.num_color_channels;
+    const PackedPixelFile& ppf, ThreadPool* pool, std::vector<uint8_t>* bytes,
+    bool encode_extra_channels, size_t extra_channel_index) const {
+  JxlExtraChannelInfo ec_info{};
+  if (encode_extra_channels) {
+    if (ppf.extra_channels_info.size() <= extra_channel_index) {
+      return JXL_FAILURE("Invalid index for extra channel");
+    }
+    ec_info = ppf.extra_channels_info[extra_channel_index].ec_info;
+  }
+
+  bool has_alpha = !encode_extra_channels && (ppf.info.alpha_bits != 0);
+  bool is_gray = encode_extra_channels || (ppf.info.num_color_channels == 1);
+  size_t color_channels =
+      encode_extra_channels ? 1 : ppf.info.num_color_channels;
   size_t num_channels = color_channels + (has_alpha ? 1 : 0);
-  size_t num_samples = num_channels * xsize * ysize;
 
   if (!ppf.info.have_animation && ppf.frames.size() != 1) {
     return JXL_FAILURE("Invalid number of frames");
@@ -284,11 +306,27 @@ Status APNGEncoder::EncodePackedPixelFileToAPNG(
   size_t anim_chunks = 0;
 
   for (const auto& frame : ppf.frames) {
-    JXL_RETURN_IF_ERROR(VerifyPackedImage(frame.color, ppf.info));
+    const PackedImage& color = encode_extra_channels
+                                   ? frame.extra_channels[extra_channel_index]
+                                   : frame.color;
 
-    const PackedImage& color = frame.color;
+    size_t xsize = color.xsize;
+    size_t ysize = color.ysize;
+    size_t num_samples = num_channels * xsize * ysize;
+
+    uint32_t bits_per_sample = encode_extra_channels ? ec_info.bits_per_sample
+                                                     : ppf.info.bits_per_sample;
+    if (!encode_extra_channels) {
+      JXL_RETURN_IF_ERROR(VerifyPackedImage(color, ppf.info));
+    } else {
+      JXL_RETURN_IF_ERROR(VerifyFormat(color.format));
+      JXL_RETURN_IF_ERROR(VerifyBitDepth(color.format.data_type,
+                                         bits_per_sample,
+                                         ec_info.exponent_bits_per_sample));
+    }
     const JxlPixelFormat format = color.format;
     const uint8_t* in = reinterpret_cast<const uint8_t*>(color.pixels());
+    JXL_RETURN_IF_ERROR(PackedImage::ValidateDataType(format.data_type));
     size_t data_bits_per_sample = PackedImage::BitsPerChannel(format.data_type);
     size_t bytes_per_sample = data_bits_per_sample / 8;
     size_t out_bytes_per_sample = bytes_per_sample > 1 ? 2 : 1;
@@ -297,40 +335,55 @@ Status APNGEncoder::EncodePackedPixelFileToAPNG(
     std::vector<uint8_t> out(out_size);
 
     if (format.data_type == JXL_TYPE_UINT8) {
-      if (ppf.info.bits_per_sample < 8) {
-        float mul = 255.0 / ((1u << ppf.info.bits_per_sample) - 1);
+      if (bits_per_sample < 8) {
+        float mul = 255.0 / ((1u << bits_per_sample) - 1);
         for (size_t i = 0; i < num_samples; ++i) {
-          out[i] = static_cast<uint8_t>(in[i] * mul + 0.5);
+          out[i] = static_cast<uint8_t>(std::lroundf(in[i] * mul));
         }
       } else {
-        memcpy(&out[0], in, out_size);
+        memcpy(out.data(), in, out_size);
       }
     } else if (format.data_type == JXL_TYPE_UINT16) {
-      if (ppf.info.bits_per_sample < 16 ||
-          format.endianness != JXL_BIG_ENDIAN) {
-        float mul = 65535.0 / ((1u << ppf.info.bits_per_sample) - 1);
+      if (bits_per_sample < 16 || format.endianness != JXL_BIG_ENDIAN) {
+        float mul = 65535.0 / ((1u << bits_per_sample) - 1);
         const uint8_t* p_in = in;
         uint8_t* p_out = out.data();
         for (size_t i = 0; i < num_samples; ++i, p_in += 2, p_out += 2) {
           uint32_t val = (format.endianness == JXL_BIG_ENDIAN ? LoadBE16(p_in)
                                                               : LoadLE16(p_in));
-          StoreBE16(static_cast<uint32_t>(val * mul + 0.5), p_out);
+          StoreBE16(static_cast<uint32_t>(std::lroundf(val * mul)), p_out);
         }
       } else {
-        memcpy(&out[0], in, out_size);
+        memcpy(out.data(), in, out_size);
+      }
+    } else if (format.data_type == JXL_TYPE_FLOAT) {
+      constexpr float kMul = 65535.0;
+      const uint8_t* p_in = in;
+      uint8_t* p_out = out.data();
+      for (size_t i = 0; i < num_samples;
+           ++i, p_in += sizeof(float), p_out += 2) {
+        float val =
+            Clamp1(format.endianness == JXL_BIG_ENDIAN ? LoadBEFloat(p_in)
+                   : format.endianness == JXL_LITTLE_ENDIAN
+                       ? LoadLEFloat(p_in)
+                       : *reinterpret_cast<const float*>(p_in),
+                   0.f, 1.f);
+        StoreBE16(static_cast<uint32_t>(std::lroundf(val * kMul)), p_out);
       }
     }
     png_structp png_ptr;
     png_infop info_ptr;
 
-    png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
+    png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, nullptr, nullptr,
+                                      nullptr);
 
     if (!png_ptr) return JXL_FAILURE("Could not init png encoder");
 
     info_ptr = png_create_info_struct(png_ptr);
     if (!info_ptr) return JXL_FAILURE("Could not init png info struct");
+    png_set_compression_level(png_ptr, 1);
 
-    png_set_write_fn(png_ptr, bytes, PngWrite, NULL);
+    png_set_write_fn(png_ptr, bytes, PngWrite, nullptr);
     png_set_flush(png_ptr, 0);
 
     int width = xsize;
@@ -343,9 +396,12 @@ Status APNGEncoder::EncodePackedPixelFileToAPNG(
     png_set_IHDR(png_ptr, info_ptr, width, height, bit_depth, color_type,
                  PNG_INTERLACE_NONE, PNG_COMPRESSION_TYPE_BASE,
                  PNG_FILTER_TYPE_BASE);
-    if (count == 0) {
+    if (count == 0 && !encode_extra_channels) {
       if (!MaybeAddSRGB(ppf.color_encoding, png_ptr, info_ptr)) {
-        MaybeAddCICP(ppf.color_encoding, png_ptr, info_ptr);
+        if (ppf.primary_color_representation !=
+            PackedPixelFile::kIccIsPrimary) {
+          MaybeAddCICP(ppf.color_encoding, png_ptr, info_ptr);
+        }
         if (!ppf.icc.empty()) {
           png_set_benign_errors(png_ptr, 1);
           png_set_iCCP(png_ptr, info_ptr, "1", 0, ppf.icc.data(),
@@ -406,7 +462,7 @@ Status APNGEncoder::EncodePackedPixelFileToAPNG(
 
     png_write_flush(png_ptr);
     const size_t pos = bytes->size();
-    png_write_image(png_ptr, &rows[0]);
+    png_write_image(png_ptr, rows.data());
     png_write_flush(png_ptr);
     if (count > 0) {
       std::vector<uint8_t> fdata(4);
@@ -414,10 +470,10 @@ Status APNGEncoder::EncodePackedPixelFileToAPNG(
       size_t p = pos;
       while (p + 8 < bytes->size()) {
         size_t len = png_get_uint_32(bytes->data() + p);
-        JXL_ASSERT(bytes->operator[](p + 4) == 'I');
-        JXL_ASSERT(bytes->operator[](p + 5) == 'D');
-        JXL_ASSERT(bytes->operator[](p + 6) == 'A');
-        JXL_ASSERT(bytes->operator[](p + 7) == 'T');
+        JXL_ENSURE(bytes->operator[](p + 4) == 'I');
+        JXL_ENSURE(bytes->operator[](p + 5) == 'D');
+        JXL_ENSURE(bytes->operator[](p + 6) == 'A');
+        JXL_ENSURE(bytes->operator[](p + 7) == 'T');
         fdata.insert(fdata.end(), bytes->data() + p + 8,
                      bytes->data() + p + 8 + len);
         p += len + 12;
@@ -430,7 +486,7 @@ Status APNGEncoder::EncodePackedPixelFileToAPNG(
 
     count++;
     if (count == ppf.frames.size() || !ppf.info.have_animation) {
-      png_write_end(png_ptr, NULL);
+      png_write_end(png_ptr, nullptr);
     }
 
     png_destroy_write_struct(&png_ptr, &info_ptr);
