@@ -8,80 +8,48 @@
 
 // Error handling: Status return type + helper macros.
 
-#include <stdarg.h>
-#include <stdint.h>
-#include <stdio.h>
-#include <stdlib.h>
-
+#include <cstdarg>
+#include <cstdint>
+#include <cstdio>
+#include <cstdlib>
 #include <type_traits>
 #include <utility>
 
+#include "lib/jxl/base/common.h"
 #include "lib/jxl/base/compiler_specific.h"
-#include "lib/jxl/base/sanitizer_definitions.h"
-
-#if JXL_ADDRESS_SANITIZER || JXL_MEMORY_SANITIZER || JXL_THREAD_SANITIZER
-#include "sanitizer/common_interface_defs.h"  // __sanitizer_print_stack_trace
-#endif                                        // defined(*_SANITIZER)
 
 namespace jxl {
 
-// Uncomment to abort when JXL_FAILURE or JXL_STATUS with a fatal error is
-// reached:
-// #define JXL_CRASH_ON_ERROR
-
-#ifndef JXL_ENABLE_ASSERT
-#define JXL_ENABLE_ASSERT 1
-#endif
-
-#ifndef JXL_ENABLE_CHECK
-#define JXL_ENABLE_CHECK 1
-#endif
-
-// Pass -DJXL_DEBUG_ON_ERROR at compile time to print debug messages when a
-// function returns JXL_FAILURE or calls JXL_NOTIFY_ERROR. Note that this is
-// irrelevant if you also pass -DJXL_CRASH_ON_ERROR.
-#if defined(JXL_DEBUG_ON_ERROR) || defined(JXL_CRASH_ON_ERROR)
-#undef JXL_DEBUG_ON_ERROR
-#define JXL_DEBUG_ON_ERROR 1
-#else  // JXL_DEBUG_ON_ERROR || JXL_CRASH_ON_ERROR
-#ifdef NDEBUG
-#define JXL_DEBUG_ON_ERROR 0
-#else  // NDEBUG
-#define JXL_DEBUG_ON_ERROR 1
-#endif  // NDEBUG
-#endif  // JXL_DEBUG_ON_ERROR || JXL_CRASH_ON_ERROR
-
-// Pass -DJXL_DEBUG_ON_ALL_ERROR at compile time to print debug messages on
-// all error (fatal and non-fatal) status. This implies JXL_DEBUG_ON_ERROR.
-#if defined(JXL_DEBUG_ON_ALL_ERROR)
-#undef JXL_DEBUG_ON_ALL_ERROR
-#define JXL_DEBUG_ON_ALL_ERROR 1
-// JXL_DEBUG_ON_ALL_ERROR implies JXL_DEBUG_ON_ERROR too.
-#undef JXL_DEBUG_ON_ERROR
-#define JXL_DEBUG_ON_ERROR 1
-#else  // JXL_DEBUG_ON_ALL_ERROR
-#define JXL_DEBUG_ON_ALL_ERROR 0
-#endif  // JXL_DEBUG_ON_ALL_ERROR
-
 // The Verbose level for the library
 #ifndef JXL_DEBUG_V_LEVEL
 #define JXL_DEBUG_V_LEVEL 0
 #endif  // JXL_DEBUG_V_LEVEL
 
-// Pass -DJXL_DEBUG_ON_ABORT={0,1} to force disable/enable the debug messages on
-// JXL_ASSERT, JXL_CHECK and JXL_ABORT.
-#ifndef JXL_DEBUG_ON_ABORT
-#define JXL_DEBUG_ON_ABORT JXL_DEBUG_ON_ERROR
-#endif  // JXL_DEBUG_ON_ABORT
+#ifdef USE_ANDROID_LOGGER
+#include <android/log.h>
+#define LIBJXL_ANDROID_LOG_TAG ("libjxl")
+inline void android_vprintf(const char* format, va_list args) {
+  char* message = nullptr;
+  int res = vasprintf(&message, format, args);
+  if (res != -1) {
+    __android_log_write(ANDROID_LOG_DEBUG, LIBJXL_ANDROID_LOG_TAG, message);
+    free(message);
+  }
+}
+#endif
 
-// Print a debug message on standard error. You should use the JXL_DEBUG macro
-// instead of calling Debug directly. This function returns false, so it can be
-// used as a return value in JXL_FAILURE.
+// Print a debug message on standard error or android logs. You should use the
+// JXL_DEBUG macro instead of calling Debug directly. This function returns
+// false, so it can be used as a return value in JXL_FAILURE.
 JXL_FORMAT(1, 2)
 inline JXL_NOINLINE bool Debug(const char* format, ...) {
   va_list args;
   va_start(args, format);
+#ifdef USE_ANDROID_LOGGER
+  android_vprintf(format, args);
+#else
   vfprintf(stderr, format, args);
+#endif
   va_end(args);
   return false;
 }
@@ -110,90 +78,52 @@ inline JXL_NOINLINE bool Debug(const char* format, ...) {
 // JXL_DEBUG version that prints the debug message if the global verbose level
 // defined at compile time by JXL_DEBUG_V_LEVEL is greater or equal than the
 // passed level.
+#if JXL_DEBUG_V_LEVEL > 0
 #define JXL_DEBUG_V(level, format, ...) \
   JXL_DEBUG(level <= JXL_DEBUG_V_LEVEL, format, ##__VA_ARGS__)
+#else
+#define JXL_DEBUG_V(level, format, ...)
+#endif
 
-// Warnings (via JXL_WARNING) are enabled by default in debug builds (opt and
-// debug).
-#ifdef JXL_DEBUG_WARNING
-#undef JXL_DEBUG_WARNING
-#define JXL_DEBUG_WARNING 1
-#else  // JXL_DEBUG_WARNING
-#ifdef NDEBUG
-#define JXL_DEBUG_WARNING 0
-#else  // JXL_DEBUG_WARNING
-#define JXL_DEBUG_WARNING 1
-#endif  // NDEBUG
-#endif  // JXL_DEBUG_WARNING
 #define JXL_WARNING(format, ...) \
-  JXL_DEBUG(JXL_DEBUG_WARNING, format, ##__VA_ARGS__)
+  JXL_DEBUG(JXL_IS_DEBUG_BUILD, format, ##__VA_ARGS__)
 
+#if JXL_IS_DEBUG_BUILD
 // Exits the program after printing a stack trace when possible.
 JXL_NORETURN inline JXL_NOINLINE bool Abort() {
-#if JXL_ADDRESS_SANITIZER || JXL_MEMORY_SANITIZER || JXL_THREAD_SANITIZER
-  // If compiled with any sanitizer print a stack trace. This call doesn't crash
-  // the program, instead the trap below will crash it also allowing gdb to
-  // break there.
-  __sanitizer_print_stack_trace();
-#endif  // *_SANITIZER)
-
-#if JXL_COMPILER_MSVC
-  __debugbreak();
-  abort();
-#else
-  __builtin_trap();
-#endif
+  JXL_PRINT_STACK_TRACE();
+  JXL_CRASH();
 }
+#endif
 
-// Exits the program after printing file/line plus a formatted string.
-#define JXL_ABORT(format, ...)                                              \
-  ((JXL_DEBUG_ON_ABORT) && ::jxl::Debug(("%s:%d: JXL_ABORT: " format "\n"), \
-                                        __FILE__, __LINE__, ##__VA_ARGS__), \
-   ::jxl::Abort())
+#if JXL_IS_DEBUG_BUILD
+#define JXL_DEBUG_ABORT(format, ...)                                   \
+  do {                                                                 \
+    if (JXL_DEBUG_ON_ABORT) {                                          \
+      ::jxl::Debug(("%s:%d: JXL_DEBUG_ABORT: " format "\n"), __FILE__, \
+                   __LINE__, ##__VA_ARGS__);                           \
+    }                                                                  \
+    ::jxl::Abort();                                                    \
+  } while (0);
+#else
+#define JXL_DEBUG_ABORT(format, ...)
+#endif
 
 // Use this for code paths that are unreachable unless the code would change
 // to make it reachable, in which case it will print a warning and abort in
 // debug builds. In release builds no code is produced for this, so only use
 // this if this path is really unreachable.
-#define JXL_UNREACHABLE(format, ...)                                   \
-  do {                                                                 \
-    if (JXL_DEBUG_WARNING) {                                           \
-      ::jxl::Debug(("%s:%d: JXL_UNREACHABLE: " format "\n"), __FILE__, \
-                   __LINE__, ##__VA_ARGS__);                           \
-      ::jxl::Abort();                                                  \
-    } else {                                                           \
-      JXL_UNREACHABLE_BUILTIN;                                         \
-    }                                                                  \
-  } while (0)
-
-// Does not guarantee running the code, use only for debug mode checks.
-#if JXL_ENABLE_ASSERT
-#define JXL_ASSERT(condition)                                      \
-  do {                                                             \
-    if (!(condition)) {                                            \
-      JXL_DEBUG(JXL_DEBUG_ON_ABORT, "JXL_ASSERT: %s", #condition); \
-      ::jxl::Abort();                                              \
-    }                                                              \
-  } while (0)
-#else
-#define JXL_ASSERT(condition) \
-  do {                        \
-  } while (0)
+#if JXL_IS_DEBUG_BUILD
+#define JXL_UNREACHABLE(format, ...)                                          \
+  (::jxl::Debug(("%s:%d: JXL_UNREACHABLE: " format "\n"), __FILE__, __LINE__, \
+                ##__VA_ARGS__),                                               \
+   ::jxl::Abort(), JXL_FAILURE(format, ##__VA_ARGS__))
+#else  // JXL_IS_DEBUG_BUILD
+#define JXL_UNREACHABLE(format, ...) \
+  JXL_FAILURE("internal: " format, ##__VA_ARGS__)
 #endif
 
-// Define JXL_IS_DEBUG_BUILD that denotes asan, msan and other debug builds,
-// but not opt or release.
-#ifndef JXL_IS_DEBUG_BUILD
-#if !defined(NDEBUG) || defined(ADDRESS_SANITIZER) ||         \
-    defined(MEMORY_SANITIZER) || defined(THREAD_SANITIZER) || \
-    defined(__clang_analyzer__)
-#define JXL_IS_DEBUG_BUILD 1
-#else
-#define JXL_IS_DEBUG_BUILD 0
-#endif
-#endif  //  JXL_IS_DEBUG_BUILD
-
-// Same as above, but only runs in debug builds (builds where NDEBUG is not
+// Only runs in debug builds (builds where NDEBUG is not
 // defined). This is useful for slower asserts that we want to run more rarely
 // than usual. These will run on asan, msan and other debug builds, but not in
 // opt or release.
@@ -206,25 +136,7 @@ JXL_NORETURN inline JXL_NOINLINE bool Abort() {
     }                                                               \
   } while (0)
 #else
-#define JXL_DASSERT(condition) \
-  do {                         \
-  } while (0)
-#endif
-
-// Always runs the condition, so can be used for non-debug calls.
-#if JXL_ENABLE_CHECK
-#define JXL_CHECK(condition)                                      \
-  do {                                                            \
-    if (!(condition)) {                                           \
-      JXL_DEBUG(JXL_DEBUG_ON_ABORT, "JXL_CHECK: %s", #condition); \
-      ::jxl::Abort();                                             \
-    }                                                             \
-  } while (0)
-#else
-#define JXL_CHECK(condition) \
-  do {                       \
-    (void)(condition);       \
-  } while (0)
+#define JXL_DASSERT(condition)
 #endif
 
 // A jxl::Status value from a StatusCode or Status which prints a debug message
@@ -273,6 +185,25 @@ JXL_NORETURN inline JXL_NOINLINE bool Abort() {
     }                                                    \
   } while (0)
 
+#if JXL_IS_DEBUG_BUILD
+// Debug: fatal check.
+#define JXL_ENSURE(condition)                     \
+  do {                                            \
+    if (!(condition)) {                           \
+      ::jxl::Debug("JXL_ENSURE: %s", #condition); \
+      ::jxl::Abort();                             \
+    }                                             \
+  } while (0)
+#else
+// Release: non-fatal check of condition. If false, just return an error.
+#define JXL_ENSURE(condition)                           \
+  do {                                                  \
+    if (!(condition)) {                                 \
+      return JXL_FAILURE("JXL_ENSURE: %s", #condition); \
+    }                                                   \
+  } while (0)
+#endif
+
 enum class StatusCode : int32_t {
   // Non-fatal errors (negative values).
   kNotEnoughBytes = -1,
@@ -323,19 +254,22 @@ static constexpr Status OkStatus() { return Status(StatusCode::kOk); }
 // needed.
 inline JXL_FORMAT(2, 3) Status
     StatusMessage(const Status status, const char* format, ...) {
-  // This block will be optimized out when JXL_DEBUG_ON_ERROR and
-  // JXL_DEBUG_ON_ALL_ERROR are both disabled.
-  if ((JXL_DEBUG_ON_ERROR && status.IsFatalError()) ||
+  // This block will be optimized out when JXL_IS_DEBUG_BUILD is disabled.
+  if ((JXL_IS_DEBUG_BUILD && status.IsFatalError()) ||
       (JXL_DEBUG_ON_ALL_ERROR && !status)) {
     va_list args;
     va_start(args, format);
+#ifdef USE_ANDROID_LOGGER
+    android_vprintf(format, args);
+#else
     vfprintf(stderr, format, args);
+#endif
     va_end(args);
   }
-#ifdef JXL_CRASH_ON_ERROR
+#if JXL_CRASH_ON_ERROR
   // JXL_CRASH_ON_ERROR means to Abort() only on non-fatal errors.
   if (status.IsFatalError()) {
-    Abort();
+    ::jxl::Abort();
   }
 #endif  // JXL_CRASH_ON_ERROR
   return status;
@@ -354,7 +288,7 @@ class JXL_MUST_USE_RESULT StatusOr {
  public:
   // NOLINTNEXTLINE(google-explicit-constructor)
   StatusOr(StatusCode code) : code_(code) {
-    JXL_ASSERT(code_ != StatusCode::kOk);
+    JXL_DASSERT(code_ != StatusCode::kOk);
   }
 
   // NOLINTNEXTLINE(google-explicit-constructor)
@@ -392,9 +326,9 @@ class JXL_MUST_USE_RESULT StatusOr {
   Status status() const { return code_; }
 
   // Only call this if you are absolutely sure that `ok()` is true.
-  // Ideally, never call this manually and rely on JXL_ASSIGN_OR_RETURN.
-  T value() && {
-    JXL_ASSERT(ok());
+  // Never call this manually: rely on JXL_ASSIGN_OR.
+  T value_() && {
+    JXL_DASSERT(ok());
     return std::move(storage_.data_);
   }
 
@@ -417,13 +351,27 @@ class JXL_MUST_USE_RESULT StatusOr {
 
 #define JXL_ASSIGN_OR_RETURN(lhs, statusor) \
   PRIVATE_JXL_ASSIGN_OR_RETURN_IMPL(        \
-      assign_or_return_temporary_variable##__LINE__, lhs, statusor)
+      JXL_JOIN(assign_or_return_temporary_variable, __LINE__), lhs, statusor)
 
 // NOLINTBEGIN(bugprone-macro-parentheses)
 #define PRIVATE_JXL_ASSIGN_OR_RETURN_IMPL(name, lhs, statusor) \
   auto name = statusor;                                        \
   JXL_RETURN_IF_ERROR(name.status());                          \
-  lhs = std::move(name).value();
+  lhs = std::move(name).value_();
+// NOLINTEND(bugprone-macro-parentheses)
+
+#define JXL_ASSIGN_OR_QUIT(lhs, statusor, message)                     \
+  PRIVATE_JXL_ASSIGN_OR_QUIT_IMPL(                                     \
+      JXL_JOIN(assign_or_temporary_variable, __LINE__), lhs, statusor, \
+      message)
+
+// NOLINTBEGIN(bugprone-macro-parentheses)
+#define PRIVATE_JXL_ASSIGN_OR_QUIT_IMPL(name, lhs, statusor, message) \
+  auto name = statusor;                                               \
+  if (!name.ok()) {                                                   \
+    QUIT(message);                                                    \
+  }                                                                   \
+  lhs = std::move(name).value_();
 // NOLINTEND(bugprone-macro-parentheses)
 
 }  // namespace jxl
