@@ -6,21 +6,25 @@
 #include <jxl/codestream_header.h>
 #include <jxl/decode.h>
 #include <jxl/decode_cxx.h>
+#include <jxl/memory_manager.h>
 #include <jxl/thread_parallel_runner.h>
 #include <jxl/thread_parallel_runner_cxx.h>
 #include <jxl/types.h>
-#include <limits.h>
-#include <stdint.h>
-#include <stdlib.h>
-#include <string.h>
 
 #include <algorithm>
+#include <cstdint>
+#include <cstdlib>
+#include <cstring>
 #include <hwy/targets.h>
 #include <map>
 #include <mutex>
 #include <random>
 #include <vector>
 
+#include "lib/jxl/base/compiler_specific.h"
+#include "lib/jxl/fuzztest.h"
+#include "tools/tracking_memory_manager.h"
+
 namespace {
 
 // Externally visible value to ensure pixels are used in the fuzzer.
@@ -28,6 +32,15 @@ int external_code = 0;
 
 constexpr const size_t kStreamingTargetNumberOfChunks = 128;
 
+using ::jpegxl::tools::kGiB;
+using ::jpegxl::tools::TrackingMemoryManager;
+
+void Check(bool ok) {
+  if (!ok) {
+    JXL_CRASH();
+  }
+}
+
 // Options for the fuzzing
 struct FuzzSpec {
   JxlDataType output_type;
@@ -65,7 +78,8 @@ void Consume(const T& entry) {
 
 // use_streaming: if true, decodes the data in small chunks, if false, decodes
 // it in one shot.
-bool DecodeJpegXl(const uint8_t* jxl, size_t size, size_t max_pixels,
+bool DecodeJpegXl(const uint8_t* jxl, size_t size,
+                  JxlMemoryManager* memory_manager, size_t max_pixels,
                   const FuzzSpec& spec, std::vector<uint8_t>* pixels,
                   std::vector<uint8_t>* jpeg, size_t* xsize, size_t* ysize,
                   std::vector<uint8_t>* icc_profile) {
@@ -73,12 +87,12 @@ bool DecodeJpegXl(const uint8_t* jxl, size_t size, size_t max_pixels,
   // itself is already multithreaded.
   size_t num_threads =
       std::min<size_t>(2, JxlThreadParallelRunnerDefaultNumWorkerThreads());
-  auto runner = JxlThreadParallelRunnerMake(nullptr, num_threads);
+  auto runner = JxlThreadParallelRunnerMake(memory_manager, num_threads);
 
   std::mt19937 mt(spec.random_seed);
   std::exponential_distribution<> dis_streaming(kStreamingTargetNumberOfChunks);
 
-  auto dec = JxlDecoderMake(nullptr);
+  auto dec = JxlDecoderMake(memory_manager);
   if (JXL_DEC_SUCCESS !=
       JxlDecoderSubscribeEvents(
           dec.get(), JXL_DEC_BASIC_INFO | JXL_DEC_COLOR_ENCODING |
@@ -511,7 +525,7 @@ bool DecodeJpegXl(const uint8_t* jxl, size_t size, size_t max_pixels,
   }
 }
 
-int TestOneInput(const uint8_t* data, size_t size) {
+int DoTestOneInput(const uint8_t* data, size_t size) {
   if (size < 4) return 0;
   uint32_t flags = 0;
   size_t used_flag_bits = 0;
@@ -556,11 +570,14 @@ int TestOneInput(const uint8_t* data, size_t size) {
   size_t ysize;
   size_t max_pixels = 1 << 21;
 
+  TrackingMemoryManager memory_manager{/* cap */ 1 * kGiB,
+                                       /* total_cap */ 5 * kGiB};
   const auto targets = hwy::SupportedAndGeneratedTargets();
   hwy::SetSupportedTargetsForTest(targets[getFlag(targets.size() - 1)]);
-  DecodeJpegXl(data, size, max_pixels, spec, &pixels, &jpeg, &xsize, &ysize,
-               &icc);
+  DecodeJpegXl(data, size, memory_manager.get(), max_pixels, spec, &pixels,
+               &jpeg, &xsize, &ysize, &icc);
   hwy::SetSupportedTargetsForTest(0);
+  Check(memory_manager.Reset());
 
   return 0;
 }
@@ -568,5 +585,11 @@ int TestOneInput(const uint8_t* data, size_t size) {
 }  // namespace
 
 extern "C" int LLVMFuzzerTestOneInput(const uint8_t* data, size_t size) {
-  return TestOneInput(data, size);
+  return DoTestOneInput(data, size);
 }
+
+void TestOneInput(const std::vector<uint8_t>& data) {
+  DoTestOneInput(data.data(), data.size());
+}
+
+FUZZ_TEST(DjxlFuzzTest, TestOneInput);
